@@ -2,7 +2,7 @@
  * wordperfect file (and styles to go along with them) and writes them
  * to a Writer target document
  *
- * Copyright (C) 2002-2004 William Lachance (william.lachance@sympatico.ca)
+ * Copyright (C) 2002-2004 William Lachance (wrlach@gmail.com)
  * Copyright (C) 2003-2004 Net Integration Technologies (http://www.net-itech.com)
  * Copyright (C) 2004 Fridrich Strba (fridrich.strba@bluewin.ch)
  *
@@ -66,8 +66,8 @@ WordPerfectCollector::WordPerfectCollector(WPXInputStream *pInput, DocumentHandl
 	miLastListLevel(0),
 	miLastListNumber(0),
 	mbListContinueNumbering(false),
-	mbListElementParagraphOpened(false),
-	mbListElementOpened(false)
+	mbListElementOpened(false),
+	mbListElementParagraphOpened(false)
 {
 }
 
@@ -88,7 +88,7 @@ bool WordPerfectCollector::filter()
         // WLACH_REFACTORING: Remove these args..
  	if (!_parseSourceDocument(*mpInput))
 		return false;
-	if (!_writeTargetDocument(*mpHandler))
+	if (!_writeTargetDocument(mpHandler))
 		return false;
 
  	// clean up the mess we made
@@ -96,7 +96,7 @@ bool WordPerfectCollector::filter()
 
 	WRITER_DEBUG_MSG(("Destroying the body elements\n"));
 	for (std::vector<DocumentElement *>::iterator iterBody = mBodyElements.begin(); iterBody != mBodyElements.end(); iterBody++) {
-		delete((*iterBody));
+		delete (*iterBody);
 		(*iterBody) = NULL;
 	}
 
@@ -109,24 +109,28 @@ bool WordPerfectCollector::filter()
 
 	WRITER_DEBUG_MSG(("Destroying the rest of the styles elements\n"));
 	for (std::map<WPXString, ParagraphStyle *, ltstr>::iterator iterTextStyle = mTextStyleHash.begin(); iterTextStyle != mTextStyleHash.end(); iterTextStyle++) {
-		delete(iterTextStyle->second);
+		delete (iterTextStyle->second);
 	}
+	for (std::map<WPXString, SpanStyle *, ltstr>::iterator iterSpanStyle = mSpanStyleHash.begin(); iterSpanStyle != mSpanStyleHash.end(); iterSpanStyle++) {
+		delete(iterSpanStyle->second);
+	}
+	
 	for (std::map<WPXString, FontStyle *, ltstr>::iterator iterFont = mFontHash.begin(); iterFont != mFontHash.end(); iterFont++) {
 		delete(iterFont->second);
 	}
 
 	for (std::vector<ListStyle *>::iterator iterListStyles = mListStyles.begin(); iterListStyles != mListStyles.end(); iterListStyles++) {
-		delete((*iterListStyles));
+		delete(*iterListStyles);
 	}
 	for (std::vector<SectionStyle *>::iterator iterSectionStyles = mSectionStyles.begin(); iterSectionStyles != mSectionStyles.end(); iterSectionStyles++) {
-		delete((*iterSectionStyles));
+		delete(*iterSectionStyles);
 	}
 	for (std::vector<TableStyle *>::iterator iterTableStyles = mTableStyles.begin(); iterTableStyles != mTableStyles.end(); iterTableStyles++) {
 		delete((*iterTableStyles));
 	}
 
 	for (std::vector<PageSpan *>::iterator iterPageSpans = mPageSpans.begin(); iterPageSpans != mPageSpans.end(); iterPageSpans++) {
-		delete((*iterPageSpans));
+		delete(*iterPageSpans);
 	}
 
  	return true;
@@ -141,62 +145,62 @@ bool WordPerfectCollector::_parseSourceDocument(WPXInputStream &input)
 	return true;
 }
 
-void WordPerfectCollector::_writeDefaultStyles(DocumentHandler &xHandler)
+void WordPerfectCollector::_writeDefaultStyles(DocumentHandler *pHandler)
 {
 	TagOpenElement stylesOpenElement("office:styles");
-	stylesOpenElement.write(xHandler);
+	stylesOpenElement.write(pHandler);
 
 	TagOpenElement defaultParagraphStyleOpenElement("style:default-style");
 	defaultParagraphStyleOpenElement.addAttribute("style:family", "paragraph");
-	defaultParagraphStyleOpenElement.write(xHandler);
+	defaultParagraphStyleOpenElement.write(pHandler);
 
 	TagOpenElement defaultParagraphStylePropertiesOpenElement("style:properties");
 	defaultParagraphStylePropertiesOpenElement.addAttribute("style:family", "paragraph");
 	defaultParagraphStylePropertiesOpenElement.addAttribute("style:tab-stop-distance", "0.5inch");
-	defaultParagraphStylePropertiesOpenElement.write(xHandler);
+	defaultParagraphStylePropertiesOpenElement.write(pHandler);
 	TagCloseElement defaultParagraphStylePropertiesCloseElement("style:properties");
-	defaultParagraphStylePropertiesCloseElement.write(xHandler);
+	defaultParagraphStylePropertiesCloseElement.write(pHandler);
 
 	TagCloseElement defaultParagraphStyleCloseElement("style:default-style");
-	defaultParagraphStyleCloseElement.write(xHandler);
+	defaultParagraphStyleCloseElement.write(pHandler);
 	
 	TagOpenElement standardStyleOpenElement("style:style");
         standardStyleOpenElement.addAttribute("style:name", "Standard");
         standardStyleOpenElement.addAttribute("style:family", "paragraph");
         standardStyleOpenElement.addAttribute("style:class", "text");
-        standardStyleOpenElement.write(xHandler);
+        standardStyleOpenElement.write(pHandler);
         TagCloseElement standardStyleCloseElement("style:style");
-        standardStyleCloseElement.write(xHandler);
+        standardStyleCloseElement.write(pHandler);
 
         TagOpenElement textBodyStyleOpenElement("style:style");
         textBodyStyleOpenElement.addAttribute("style:name", "Text Body");
         textBodyStyleOpenElement.addAttribute("style:family", "paragraph");
         textBodyStyleOpenElement.addAttribute("style:parent-style-name", "Standard");
         textBodyStyleOpenElement.addAttribute("style:class", "text");
-        textBodyStyleOpenElement.write(xHandler);
+        textBodyStyleOpenElement.write(pHandler);
         TagCloseElement textBodyStyleCloseElement("style:style");
-        textBodyStyleCloseElement.write(xHandler);
+        textBodyStyleCloseElement.write(pHandler);
 
         TagOpenElement tableContentsStyleOpenElement("style:style");
         tableContentsStyleOpenElement.addAttribute("style:name", "Table Contents");
         tableContentsStyleOpenElement.addAttribute("style:family", "paragraph");
         tableContentsStyleOpenElement.addAttribute("style:parent-style-name", "Text Body");
         tableContentsStyleOpenElement.addAttribute("style:class", "extra");
-        tableContentsStyleOpenElement.write(xHandler);
+        tableContentsStyleOpenElement.write(pHandler);
         TagCloseElement tableContentsStyleCloseElement("style:style");
-        tableContentsStyleCloseElement.write(xHandler);
+        tableContentsStyleCloseElement.write(pHandler);
 
         TagOpenElement tableHeadingStyleOpenElement("style:style");
         tableHeadingStyleOpenElement.addAttribute("style:name", "Table Heading");
         tableHeadingStyleOpenElement.addAttribute("style:family", "paragraph");
         tableHeadingStyleOpenElement.addAttribute("style:parent-style-name", "Table Contents");
         tableHeadingStyleOpenElement.addAttribute("style:class", "extra");
-        tableHeadingStyleOpenElement.write(xHandler);
+        tableHeadingStyleOpenElement.write(pHandler);
         TagCloseElement tableHeadingStyleCloseElement("style:style");
-        tableHeadingStyleCloseElement.write(xHandler);
+        tableHeadingStyleCloseElement.write(pHandler);
 
 	TagCloseElement stylesCloseElement("office:styles");
-	stylesCloseElement.write(xHandler);
+	stylesCloseElement.write(pHandler);
 
 }
 
@@ -205,32 +209,31 @@ void WordPerfectCollector::_writeBegin()
 {
 }
 
-void WordPerfectCollector::_writeMasterPages(DocumentHandler &xHandler)
+void WordPerfectCollector::_writeMasterPages(DocumentHandler *pHandler)
 {
         WPXPropertyList xBlankAttrList;
 
-	xHandler.startElement("office:master-styles", xBlankAttrList);
+	pHandler->startElement("office:master-styles", xBlankAttrList);
 	int pageNumber = 1;
-	for (int i=0; i<mPageSpans.size(); i++)
+	for (unsigned int i=0; i<mPageSpans.size(); i++)
 	{
 		bool bLastPage;
 		(i == (mPageSpans.size() - 1)) ? bLastPage = true : bLastPage = false;
-		mPageSpans[i]->writeMasterPages(pageNumber, i, bLastPage, xHandler);
+		mPageSpans[i]->writeMasterPages(pageNumber, i, bLastPage, pHandler);
 		pageNumber += mPageSpans[i]->getSpan();
 	}
-	xHandler.endElement("office:master-styles");
+	pHandler->endElement("office:master-styles");
 }
 
-void WordPerfectCollector::_writePageMasters(DocumentHandler &xHandler)
+void WordPerfectCollector::_writePageMasters(DocumentHandler *pHandler)
 {
-	int pageNumber = 1;
-	for (int i=0; i<mPageSpans.size(); i++)
+	for (unsigned int i=0; i<mPageSpans.size(); i++)
 	{
-		mPageSpans[i]->writePageMaster(i, xHandler);
+		mPageSpans[i]->writePageMaster(i, pHandler);
 	}
 }
 
-bool WordPerfectCollector::_writeTargetDocument(DocumentHandler &xHandler)
+bool WordPerfectCollector::_writeTargetDocument(DocumentHandler *pHandler)
 {        
 	WRITER_DEBUG_MSG(("WriterWordPerfect: Document Body: Printing out the header stuff..\n"));
 	WPXPropertyList xBlankAttrList;
@@ -261,13 +264,13 @@ bool WordPerfectCollector::_writeTargetDocument(DocumentHandler &xHandler)
 	// write out the font styles
 	mpHandler->startElement("office:font-decls", xBlankAttrList);
 	for (std::map<WPXString, FontStyle *, ltstr>::iterator iterFont = mFontHash.begin(); iterFont != mFontHash.end(); iterFont++) {
-		iterFont->second->write(*mpHandler);
+		iterFont->second->write(mpHandler);
 	}
 	TagOpenElement symbolFontOpen("style:font-decl");
 	symbolFontOpen.addAttribute("style:name", "StarSymbol");
 	symbolFontOpen.addAttribute("fo:font-family", "StarSymbol");
 	symbolFontOpen.addAttribute("style:font-charset", "x-symbol");
-	symbolFontOpen.write(*mpHandler);
+	symbolFontOpen.write(mpHandler);
         mpHandler->endElement("style:font-decl");
 
 	mpHandler->endElement("office:font-decls");
@@ -276,7 +279,7 @@ bool WordPerfectCollector::_writeTargetDocument(DocumentHandler &xHandler)
  	WRITER_DEBUG_MSG(("WriterWordPerfect: Document Body: Writing out the styles..\n"));
 
 	// write default styles
-	_writeDefaultStyles(*mpHandler);
+	_writeDefaultStyles(mpHandler);
 
 	mpHandler->startElement("office:automatic-styles", xBlankAttrList);
 
@@ -287,7 +290,7 @@ bool WordPerfectCollector::_writeTargetDocument(DocumentHandler &xHandler)
 		if (strcmp((iterTextStyle->second)->getName().cstr(), "Standard")) 
                 {
 			// don't write standard paragraph "no styles" style
-			(iterTextStyle->second)->write(xHandler);
+			(iterTextStyle->second)->write(pHandler);
 		}
 	}
 
@@ -295,45 +298,45 @@ bool WordPerfectCollector::_writeTargetDocument(DocumentHandler &xHandler)
 	for (std::map<WPXString, SpanStyle *, ltstr>::iterator iterSpanStyle = mSpanStyleHash.begin(); 
              iterSpanStyle != mSpanStyleHash.end(); iterSpanStyle++) 
         {
-                (iterSpanStyle->second)->write(xHandler);
+                (iterSpanStyle->second)->write(pHandler);
 	}
 
  	// writing out the sections styles
 	for (std::vector<SectionStyle *>::iterator iterSectionStyles = mSectionStyles.begin(); iterSectionStyles != mSectionStyles.end(); iterSectionStyles++) {
-		(*iterSectionStyles)->write(xHandler);
+		(*iterSectionStyles)->write(pHandler);
 	}
 
 	// writing out the lists styles
 	for (std::vector<ListStyle *>::iterator iterListStyles = mListStyles.begin(); iterListStyles != mListStyles.end(); iterListStyles++) {
-		(*iterListStyles)->write(xHandler);
+		(*iterListStyles)->write(pHandler);
 	}
 
  	// writing out the table styles
 	for (std::vector<TableStyle *>::iterator iterTableStyles = mTableStyles.begin(); iterTableStyles != mTableStyles.end(); iterTableStyles++) {
-		(*iterTableStyles)->write(xHandler);
+		(*iterTableStyles)->write(pHandler);
 	}
 
 	// writing out the page masters
-	_writePageMasters(xHandler);
+	_writePageMasters(pHandler);
 
 
-	xHandler.endElement("office:automatic-styles");
+	pHandler->endElement("office:automatic-styles");
 
-	_writeMasterPages(xHandler);
+	_writeMasterPages(pHandler);
 
  	WRITER_DEBUG_MSG(("WriterWordPerfect: Document Body: Writing out the document..\n"));
  	// writing out the document
-	xHandler.startElement("office:body", xBlankAttrList);
+	pHandler->startElement("office:body", xBlankAttrList);
 
 	for (std::vector<DocumentElement *>::iterator iterBodyElements = mBodyElements.begin(); iterBodyElements != mBodyElements.end(); iterBodyElements++) {
-		(*iterBodyElements)->write(xHandler);
+		(*iterBodyElements)->write(pHandler);
 	}
  	WRITER_DEBUG_MSG(("WriterWordPerfect: Document Body: Finished writing all doc els..\n"));
 
-	xHandler.endElement("office:body");
-	xHandler.endElement("office:document-content");
+	pHandler->endElement("office:body");
+	pHandler->endElement("office:document-content");
 
-	xHandler.endDocument();
+	pHandler->endDocument();
 
 	return true;
 }
@@ -423,8 +426,14 @@ void WordPerfectCollector::closeFooter()
 void WordPerfectCollector::openSection(const WPXPropertyList &propList, const WPXPropertyListVector &columns)
 {
         int iNumColumns = columns.count();
+	float fSectionMarginLeft = 0.0f;
+	float fSectionMarginRight = 0.0f;
+	if (propList["fo:margin-left"])
+		fSectionMarginLeft = propList["fo:margin-left"]->getFloat();
+	if (propList["fo:margin-right"])
+		fSectionMarginRight = propList["fo:margin-right"]->getFloat();
 
-	if (iNumColumns > 1)
+	if (iNumColumns > 1 || fSectionMarginLeft != 0 || fSectionMarginRight != 0)
 	{
 		mfSectionSpaceAfter = propList["fo:margin-bottom"]->getFloat();
 		WPXString sSectionName;
@@ -753,7 +762,14 @@ void WordPerfectCollector::closeListElement()
 
 void WordPerfectCollector::openFootnote(const WPXPropertyList &propList)
 {
-	mpCurrentContentElements->push_back(static_cast<DocumentElement *>(new TagOpenElement("text:footnote")));
+	TagOpenElement *pOpenFootNote = new TagOpenElement("text:footnote");
+	if (propList["libwpd:number"])
+	{
+		WPXString tmpString("ftn");
+		tmpString.append(propList["libwpd:number"]->getStr());
+		pOpenFootNote->addAttribute("text:id", tmpString);
+	}
+	mpCurrentContentElements->push_back(static_cast<DocumentElement *>(pOpenFootNote));
 
 	mpCurrentContentElements->push_back(static_cast<DocumentElement *>(new TagOpenElement("text:footnote-citation")));
         if (propList["libwpd:number"])
@@ -775,7 +791,14 @@ void WordPerfectCollector::closeFootnote()
 
 void WordPerfectCollector::openEndnote(const WPXPropertyList &propList)
 {
-	mpCurrentContentElements->push_back(static_cast<DocumentElement *>(new TagOpenElement("text:endnote")));
+	TagOpenElement *pOpenEndNote = new TagOpenElement("text:endnote");
+	if (propList["libwpd:number"])
+	{
+		WPXString tmpString("edn");
+		tmpString.append(propList["libwpd:number"]->getStr());
+		pOpenEndNote->addAttribute("text:id", tmpString);
+	}
+	mpCurrentContentElements->push_back(static_cast<DocumentElement *>(pOpenEndNote));
 
 	mpCurrentContentElements->push_back(static_cast<DocumentElement *>(new TagOpenElement("text:endnote-citation")));
         if (propList["libwpd:number"])
